package sun.nio.ch;

import java.io.IOException;
import java.nio.channels.Channel;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.AsynchronousFileChannel;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;

public class FileLockImpl extends FileLock
{
    private volatile boolean valid;
    
    FileLockImpl(FileChannel channel, long position, long size, boolean shared) {
        super(channel, position, size, shared);
        this.valid = true;
    }
    
    FileLockImpl(AsynchronousFileChannel channel, long position, long size, boolean shared) {
        super(channel, position, size, shared);
        this.valid = true;
    }
    
    @Override
    public boolean isValid() {
        return this.valid;
    }
    
    void invalidate() {
        assert !(!Thread.holdsLock(this));
        this.valid = false;
        return;
    }
    
    @Override
    public synchronized void release() throws IOException {
        Channel acquiredBy = this.acquiredBy();
        if (!acquiredBy.isOpen()) {
            throw new ClosedChannelException();
        }
        if (this.valid) {
            if ((acquiredBy instanceof FileChannelImpl)) {
                ((FileChannelImpl)acquiredBy).release(this);
            }
            else if ((acquiredBy instanceof AsynchronousFileChannelImpl)) {
                ((AsynchronousFileChannelImpl)acquiredBy).release(this);
            }
            else {
                throw new AssertionError();
            }
            this.valid = false;
        }
        return;
    }
}
